<?php
/**
 * Консольный скрипт, обеспечивает загрузку дистрибутива и установку ReadyScript.
 * Скрипт должен быть размещен в корне сайта.
 * Требования для запуска: PHP 7.1+
 *
 * Описание параметров <> - обязательные [] - необязательные:
 * php installer.php [-package:package]
 *                   <-domain:domain_of_shop>
 *                   <-db_host:db_host>
 *                   <-db_name:db_name>
 *                   <-db_user:db_user>
 *                   <-db_pass:db_pass>
 *                   [-db_port:db_port]
 *                   [-db_prefix:db_port]
 *                   [-db_table_engine:db_engine]
 *                   <-supervisor_email:admin_email>
 *                   <-supervisor_pass:admin_pass>
 *                   [-admin_section:admin_section]
 *                   [-delete_self]
 *
 * Пример запуска:
 * php installer.php
 * Отобразит справку по имеющимся параметрам скрипта
 * или
 * php installer.php -package:mega -domain:example.com -db_name:rsdb -db_user:rsdbuser -db_pass:dbpassword -supervisor_email:admin@example.com -supervisor_pass:demo
 * или
 * php installer.php -package:mega -domain:example.com -db_host:localhost -db_name:rsdb -db_user:rsdbuser -db_pass:dbpassword -db_port:3306 -db_prefix:abcd -db_table_engine:MyISAM -supervisor_email:admin@example.com -supervisor_pass:demo -admin_section:admin -delete_self
 */
class Installer
{
    protected $params = [
        'package' => ['required' => false, 'title' => 'Редакция платформы, которую нужно скачать. Если не указана, то считаем, что в папке уже распакован дистрибутив', 'values' => ['base', 'middle', 'full', 'mega']],
        'domain' => ['required' => true, 'title' => 'Доменное имя магазина'],
        'db_host' => ['required' => false, 'title' => 'HOST базы данных Mysql'],
        'db_name' => ['required' => true, 'title' => 'Имя базы данных Mysql'],
        'db_user' => ['required' => true, 'title' => 'Пользователь базы данных Mysql'],
        'db_pass' => ['required' => true, 'title' => 'Пароль Пользователя базы данных Mysql'],
        'db_prefix' => ['required' => false, 'title' => 'Префикс таблиц базы данных'],
        'db_port' => ['required' => false, 'title' => 'Порт базы данных Mysql'],
        'db_table_engine' => ['required' => false, 'title' => 'Тип таблиц данных Mysql', 'values' => ['MyISAM', 'InnoDB', 'Aria']],
        'supervisor_email' => ['required' => true, 'title' => 'Email администратора'],
        'supervisor_pass' => ['required' => true, 'title' => 'Пароль администратора'],
        'admin_section' => ['required' => false, 'title' => 'Секция административной панели в URL'],
        'delete_self' => ['required' => false, 'title' => 'Удалять данный скрипт установки после успешной установки']
    ];

    protected $argv;
    protected $setupFile = '/setup.inc.php';
    protected $configAutoFile = '/config.auto.php';
    protected $appUrl = 'https://readyscript.ru/downloads/readyscript-shop-%PACKAGE%.zip';
    protected $tmpZip = '/readyscript.tmp.zip';

    //default значения конфигурации
    protected $config = [
        'db_host' => 'localhost',
        'db_name' => 'rsdb',
        'db_user' => 'dbuser',
        'db_pass' => 'dbpassword',
        'db_port' => 3306,
        'db_prefix' => '',
        'db_table_engine' => 'MyISAM',

        'supervisor_email' => 'admin@example.com',
        'supervisor_pass' => 'demo',
        'admin_section' => 'admin',

        'set_demo_data' => 1,
        'folder' => '',
        'detailed_exception' => 0,
        'default_theme' => 'amazing',
    ];

    /**
     * Installer конструктор.
     * @param array $argv Массив консольных параметров
     */
    function __construct($argv)
    {
        $this->argv = $argv;
        $alphabet = range('a', 'z');

        for($i = 1; $i < 4; $i++) {
            $this->config['db_prefix'] .= $alphabet[rand(0, count($alphabet))];
        }

        $this->setupFile = __DIR__.$this->setupFile;
        $this->configAutoFile = __DIR__.$this->configAutoFile;
        $this->tmpZip = __DIR__.$this->tmpZip;
    }

    /**
     * Запускает скрипт установки ReadyScript
     */
    public function run()
    {
        if(PHP_SAPI != 'cli') {
            echo "Данный скрипт необходимо запускать только из коммандной строки";
            exit(1);
        }

        if (count($this->argv) > 1) {
            if ($this->validate()) {
                if (isset($this->config['package'])) {
                    $this->download();
                    $this->unpack();
                }
                $this->install();
            } else {
                exit(1);
            }
        } else {
            $this->showHelp();
        }
    }

    /**
     * Загружает необходимый дистрибутив с сайта ReadyScript
     */
    protected function download()
    {
        $url = str_replace('%PACKAGE%', $this->config['package'], $this->appUrl);
        echo "Скачиваем дистрибутив... ".$url."\n";

        $arrContextOptions = stream_context_create(array(
            "ssl"=>array(
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
        ));

        if (!file_put_contents($this->tmpZip, fopen($url, 'r', null, $arrContextOptions))) {
            echo "Не удалось скачать дистрибутив\n";
            exit(1);
        }

        echo "Дистрибутив успешно загружен\n";
    }

    /**
     * Распаковывает архив, а затем удаляет его
     */
    protected function unpack()
    {
        echo "Распаковыаем дистрибутив...\n";

        $zip = new ZipArchive;
        if ($zip->open($this->tmpZip) === TRUE) {
            $zip->extractTo(__DIR__);
            $zip->close();
            echo "Распаковка прошла успешно\n";
        } else {
            echo "Ошибка распаковки дистрибутива\n";
            exit(1);
        }
    }

    /**
     * Отображает помощь по синтаксису запуска скрипта
     */
    protected function showHelp()
    {
        echo "Это скрипт-установщик ReadyScript\n";
        echo "Пример запуска <> - обязательные параметры [] - необязательные параметры: \n";

        $example_params = '';
        foreach($this->params as $key => $params_data) {
            $param_line = "-{$key}:{$this->config[$key]}";
            $example_params .= " ".($params_data['required'] ? "<{$param_line}>" : "[{$param_line}]");
        }
        echo 'php installer.php'.$example_params."\n";
        echo "\nОписание параметров: \n";

        foreach($this->params as $key => $params_data) {
            if (isset($params_data['values'])) {
                $values = '(Значения: '.implode(',', $params_data['values']).')';
            } else {
                $values = '';
            }
            echo "-{$key}:{$params_data['title']}{$values}\n";
        }
    }

    /**
     * Проверяет входящие параметры
     *
     * @return bool
     */
    protected function validate()
    {
        $errors = [];
        if (file_exists($this->configAutoFile)) {
            $errors[] = "Найден файл config.auto.php. Вероятно ReadyScript уже установлен\n";
        } else {
            foreach ($this->params as $key => $params_data) {
                $found = false;
                foreach ($this->argv as $i => $value) {
                    if ($i == 0) continue;
                    if (preg_match("/^\-{$key}(\:(.*))?$/", $value, $match)) {
                        $this->config[$key] = isset($match[2]) ? $match[2] : true;
                        $found = true;
                        break;
                    }
                }

                if ($params_data['required'] && !$found) {
                    $errors[] = "Не задан обязательный параметр: -{$key}\n";
                }

                if (isset($params_data['values']) && $found && !in_array($this->config[$key], $params_data['values'])) {
                    $errors[] = "Параметр -{$key} может иметь только следующие значения: ".implode(',', $params_data['values'])." \n";
                }
            }
        }

        echo implode("", $errors);
        return empty($errors);
    }

    /**
     * Устанавливает ReadyScript с учетом заданных параметров
     */
    protected function install()
    {
        if (!file_exists($this->setupFile)) {
            echo "Не найден файл setup.inc.php. Скрипт должен быть размещен в корне сайта, где распакован дистрибутив ReadyScript\n";
            exit(1);
        }

        require($this->setupFile);
        \Setup::$DOMAIN = $this->config['domain'];

        echo "Начало установки... \n";

        $api_install = new \Install\Model\Api();
        $start_result = $api_install->resetInstall($this->config);
        $this->displayInstallStep($start_result);

        while( ($data = $api_install->progress()) && empty($data['complete']) && empty($data['errors']) ) {
            $this->displayInstallStep($data);
        }

        $api_install->installComplete();
        $this->displayInstallStep($data);

        echo "... Установка завершена!\n";

        if (isset($this->config['delete_self'])) {
            unlink(__FILE__);
            echo "Файл установки installer.php успешно удален\n";
        }
    }

    /**
     * Отображает результат выполнения шага устаноки
     *
     * @param array $step_result Массив с результатом выполнения шага установки
     */
    protected function displayInstallStep($step_result)
    {
        if (isset($step_result['next'])) {
            echo $step_result['next']." (".$step_result['percent']."%)\n";
        } elseif (isset($step_result['errors'])) {
            foreach($step_result['errors'] as $error) {
                echo "Ошибка в модуле: {$error['moduleTitle']}: {$error['message']}\n";
            }
            exit(1);
        }
    }
}

$installer = new Installer($argv ?? []);
$installer->run();